@@ -30,7 +30,6 @@ RCDLINKS="2,S41 3,S41 6,K41"
 #	   shorewall start			  Starts the firewall
 #	   shorewall restart			  Restarts the firewall
 #	   shorewall reload			  Reload the firewall
-#						  (same as restart)
 #	   shorewall stop			  Stops the firewall
 #	   shorewall status			  Displays firewall status
 #
@@ -82,7 +81,10 @@ case "$command" in
     start)
 	exec $SBINDIR/shorewall $OPTIONS start $STARTOPTIONS
 	;;
-    restart|reload)
+    reload)
+	exec $SBINDIR/shorewall $OPTIONS reload $RELOADOPTIONS
+	;;
+    restart)
 	exec $SBINDIR/shorewall $OPTIONS restart $RESTARTOPTIONS
 	;;
     status|stop)
