@@ -10,15 +10,16 @@
 
 OPTIONS=""
 
-# Use /etc/default shorewall to specify $OPTIONS to run at startup, however this
-# this might prevent shorewall from starting. use at your own risk
+# Use /etc/default shorewall to specify $OPTIONS and STARTOPTIONS to
+# run at startup, however this this might prevent shorewall from
+# starting. use at your own risk
 if [ -f /etc/default/shorewall ] ; then
     . /etc/default/shorewall
 fi
 
 start() {
 	echo "Starting IPv4 shorewall rules..."
-	exec /sbin/shorewall $OPTIONS start
+	exec /sbin/shorewall $OPTIONS start $STARTOPTIONS
 }
 
 stop() {
@@ -26,9 +27,14 @@ stop() {
 	exec /sbin/shorewall stop
 }
 
+reload() {
+	echo "Reloading IPv4 shorewall rules..."
+	exec /sbin/shorewall reload $RELOADOPTIONS
+}
+
 restart() {
 	echo "Restarting IPv4 shorewall rules..."
-	exec /sbin/shorewall restart
+	exec /sbin/shorewall restart $RESTARTOPTIONS
 }
 
 status() {
@@ -42,10 +48,13 @@ case "$1" in
     'stop')
 		stop
 	;;
-    'reload'|'restart')
+    'reload')
+		reload
+	;;
+    'restart')
 		restart
 	;;
-	'status')
+    'status')
 		status
 	;;
     *)
